@@ -67,12 +67,24 @@ print.CA <-
 #' @export
 print.modelcomp <- function(x, ...)
 {
-  cat("\n",x$method,"statistics for items requiring two or more attributes:\n")
-  ret <- extract.modelcomp(x,"stats")
-  print(ret[,colSums(is.na(ret))==0],drop=FALSE)
-  cat("\np-values for items requiring two or more attributes:\n")
-  p <- extract.modelcomp(x,"pvalues")
-  print(p[,colSums(is.na(p))==0])
+  cat("\nItem-level model selection:\n\n")
+  if(toupper(x$method)=="LR"& x$LR.args$LR.approx){
+      m <- "Two-step approximated LR"
+    }else{
+    m <- x$method
+  }
+  cat("test statistic:",m,"\n")
+  if (x$decision.args$rule=="simpler"){
+    y <- paste0("simpler model + largest",ifelse(x$decision.args$adjusted," adjusted "," "), "p value rule")
+  }else if(x$decision.args$rule=="largestp"){
+    y <- paste0("largest" ,ifelse(x$decision.args$adjusted," adjusted "," "), "p value rule")
+  }
+  cat("Decision rule:",y,"at",x$decision.args$alpha.level,"alpha level.\n")
+  cat("Adjusted p values were based on",x$p.adjust.methods,"correction.\n\n")
+  ret <- extract(x,"selected.model")
+  ret[is.na(ret)] <- ""
+  print(ret)
+
 }
 
 #' @export
@@ -205,7 +217,10 @@ print.summary.GDINA <- function(x,...){
   # cat("Deviance      =", formatC(x$Deviance,digits = 2, format = "f"), "\n")
   cat("AIC    =", formatC(x$AIC,digits = 2, format = "f")," | penalty   =",x$`AIC Penalty`,"\n")
   cat("BIC    =", formatC(x$BIC,digits = 2, format = "f")," | penalty   =",formatC(x$`BIC penalty`,digits = 2, format = "f"),"\n")
-  cat("# par  =",formatC(x$`Number of parameters`,digits = 0, format = "d"), "\n")
+  cat("\nNo. of parameters  =",formatC(x$`Number of parameters`,digits = 0, format = "d"), "\n")
+  cat("  No. of estimated item parameters = ",formatC(x$`Number of estimated item parameters`,digits = 0, format = "d"), "\n")
+  cat("  No. of fixed item parameters = ",formatC(x$`Number of fixed item parameters`,digits = 0, format = "d"), "\n")
+  cat("  No. of distribution parameters = ",formatC(x$`Number of population parameters`,digits = 0, format = "d"), "\n")
     cat("\nAttribute Prevalence\n\n")
     ap <- lapply(x$`Attribute Prevalence`,round,digits=4)
     if(x$ngroup==1) {
@@ -258,6 +273,8 @@ print.summary.autoGDINA <- function(x,...){
 #'@export
 print.npar.GDINA <- function(x,...){
   cat("No. of total parameters =",x$`No. of parameters`,"\n")
-  cat("No. of item parameters =",x$`No. of item parameters`,"\n")
   cat("No. of population parameters =",x$`No. of population parameters`,"\n")
+  cat("No. of free item parameters =",x$`No. of estimated item parameters`,"\n")
+  cat("No. of fixed item parameters =",x$`No. of fixed item parameters`,"\n")
+
 }
