@@ -6,7 +6,7 @@ using namespace Rcpp;
 
 // [[Rcpp::export]]
 
-arma::mat aggregateCol(arma::mat mX, //N x L
+arma::mat aggregateCol(arma::mat & mX, //N x L
                    arma::vec ind){
   ind--; //indictor-1: c++ style
   arma::vec uniq = arma::unique(ind);
@@ -14,8 +14,8 @@ arma::mat aggregateCol(arma::mat mX, //N x L
   int Lj = uniq.n_elem;
   arma::mat output = arma::zeros<arma::mat>(N,Lj);
   for (int l=0;l<Lj;++l){
-    arma::uvec loc = find(ind==l);
-    output.col(l) = sum(mX.cols(loc),1); //N x 1
+    arma::uvec loc = arma::find(ind==l);
+    output.col(l) = arma::sum(mX.cols(loc),1); //N x 1
   }
 
 
